local L = LibStub("AceLocale-3.0"):GetLocale("TrackOMatic", true);

local GATHER_CUTOFFRED = 15;
local GATHER_CUTOFFGRAY = 50;

TRACKOMATIC_PROFESSION_CAPS = {
    [0] = 300,    -- Vanilla
    [1] = 375,    -- Burning Crusade
    [2] = 450,    -- Wrath of the Lich King
    [3] = 525,    -- Cataclysm
    [4] = 600,    -- Mists of Pandaria
};

--========================================
-- Update a profession bar
--========================================
function TrackOMatic_SetProfBar(data)

    -- retrieve skill info
    local profIndex, profName, profIcon, profSkillLevel, profMaxSkill, profNumAbilities, profSpelloffset, profSkillLine, profSkillModifier = TrackOMatic_GetProfessionInfo(data.id);
    if (profIndex == nil) then
        return;
    end

    -- update the bar
    local max, value, color, label, levelLabel, tooltipTitle, tooltipText, skillMod;

    label = profName;
    tooltipTitle = "";
    tooltipText = "";
    skillMod = 0;

    max = profMaxSkill;
    value = profSkillLevel;
    color = {r = 0, g = 0, b = 1};

    skillMod = profSkillLevel + profSkillModifier;

    if (profSkillModifier == 0) then
        levelLabel = profSkillLevel .. "/" .. profMaxSkill;
        tooltipTitle = profName .. " " .. profSkillLevel;
    else
        local color = RED_FONT_COLOR_CODE;
        if (profSkillModifier > 0) then
            color = GREEN_FONT_COLOR_CODE .. "+";
        end
        levelLabel = profSkillLevel .. "/" .. profMaxSkill .. " (" .. color .. profSkillModifier .. FONT_COLOR_CODE_CLOSE .. " = " .. skillMod .. ")";
        tooltipTitle = profName .. " " .. skillMod .. " (" .. profSkillLevel .. color .. profSkillModifier .. FONT_COLOR_CODE_CLOSE .. ")";
    end

    -- tooltip info
    if (profName == L["PROFESSION_SKINNING"]) then
        tooltipText = string.format(L["ABLE_TO_SKIN_LEVEL"], TrackOMatic_GetSkinnableCreatureLevel(skillMod));
    elseif (profName == L["PROFESSION_MINING"]) then
        tooltipText = tooltipText .. L["ABLE_TO_MINE"] .. "\n\n";
        tooltipText = tooltipText .. TrackOMatic_GetGatherables(profName, skillMod);
        tooltipText = tooltipText .. "\n\n" .. string.format(L["ABLE_TO_MINE_CREATURE"], TrackOMatic_GetSkinnableCreatureLevel(skillMod));
    elseif (profName == L["PROFESSION_HERBALISM"]) then
        tooltipText = tooltipText .. L["ABLE_TO_HERB"] .. "\n\n";
        tooltipText = tooltipText .. TrackOMatic_GetGatherables(profName, skillMod);
        tooltipText = tooltipText .. "\n\n" .. string.format(L["ABLE_TO_HERB_CREATURE"], TrackOMatic_GetSkinnableCreatureLevel(skillMod));
    end

    local newRankAvailable = TrackOMatic_NewProfessionRankAvailable(value, max);

    if (newRankAvailable) then
        tooltipText = tooltipText .. "\n|cff00ff00" .. L["PROFESSION_NEW_RANK_AVAILABLE"];
    end

    return value, max, color, label, levelLabel, tooltipTitle, tooltipText, profName, nil, nil, newRankAvailable;

end

--========================================
-- Get profession info by name
--========================================
function TrackOMatic_GetProfessionInfo(name)
    -- retrieve profession indexes
    local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions();
    -- convert to an array
    profArray = {prof1, prof2, archaeology, fishing, cooking, firstAid};

    for i = 1, 6, 1 do
        if (profArray[i]) then
            local profName, profIcon, profSkillLevel, profMaxSkill, profNumAbilities, profSpelloffset, profSkillLine, profSkillModifier = GetProfessionInfo(profArray[i]);
            if (string.sub(string.lower(profName), 0, string.len(name)) == string.lower(name)) then
                return profArray[i], profName, profIcon, profSkillLevel, profMaxSkill, profNumAbilities, profSpelloffset, profSkillLine, profSkillModifier;
            end
        end
    end
    return nil;
end

--========================================
-- Get the maximum level of creature able
-- to be skinned at a specified skill lvl
--========================================
function TrackOMatic_GetSkinnableCreatureLevel(skillLevel)
    if (skillLevel >= 100) then
        return math.floor(skillLevel / 5);
    else
        return math.floor(skillLevel / 10) + 10;
    end
end

--========================================
-- Generate a 'gatherable node' list for
-- mining and herbalism
--========================================
function TrackOMatic_GetGatherables(profession, skillLevel)

    local nodes = {
        [L["PROFESSION_MINING"]] = {
            {name = L["MINING_NODE_COPPER"], skill = 0},    -- skill is 0 otherwise it'd change colors one skill level late
            {name = L["MINING_NODE_TIN"], skill = 0, turnsYellow = 100},
            {name = L["MINING_NODE_SILVER"], skill = 50},
            {name = L["MINING_NODE_IRON"], skill = 100},
            {name = L["MINING_NODE_GOLD"], skill = 115},
            {name = L["MINING_NODE_MITHRIL"], skill = 150},
            {name = L["MINING_NODE_TRUESILVER"], skill = 165},
            {name = L["MINING_NODE_DARKIRON"], skill = 175},
            {name = L["MINING_NODE_SMALLTHORIUM"], skill = 200},
            {name = L["MINING_NODE_RICHTHORIUM"], skill = 215},

            {name = "\n" .. L["MINING_NODE_FELIRON"], skill = 275},
            {name = L["MINING_NODE_ADAMANTITE"], skill = 325},
            {name = L["MINING_NODE_RICHADAMANTITE"], skill = 350},
            {name = L["MINING_NODE_KHORIUM"], skill = 375},

            {name = "\n" .. L["MINING_NODE_COBALT"], skill = 350},
            {name = L["MINING_NODE_RICHCOBALT"], skill = 375},
            {name = L["MINING_NODE_SARONITE"], skill = 400},
            {name = L["MINING_NODE_RICHSARONITE"], skill = 425},
            {name = L["MINING_NODE_TITANIUM"], skill = 450},

            {name = "\n" .. L["MINING_NODE_OBSIDIUM"], skill = 425},
            {name = L["MINING_NODE_RICHOBSIDIUM"], skill = 450},
            {name = L["MINING_NODE_ELEMENTIUM"], skill = 475},
            {name = L["MINING_NODE_RICHELEMENTIUM"], skill = 500},
            {name = L["MINING_NODE_PYRITE"], skill = 525},
            {name = L["MINING_NODE_RICHPYRITE"], skill = 525},

            {name = "\n" .. L["MINING_NODE_GHOSTIRON"], skill = 500},
            {name = L["MINING_NODE_KYPARITE"], skill = 550},
            {name = L["MINING_NODE_RICHGHOSTIRON"], skill = 550},
            {name = L["MINING_NODE_RICHKYPARITE"], skill = 575},
            {name = L["MINING_NODE_TRILLIUM"], skill = 600},
            {name = L["MINING_NODE_RICHTRILLIUM"], skill = 600},
        },
        [L["PROFESSION_HERBALISM"]] = {
            {name = L["HERB_PEACEBLOOM"], skill = 0},       -- see copper
            {name = L["HERB_SILVERLEAF"], skill = 0},       -- ...
            {name = L["HERB_EARTHROOT"], skill = 0},
            {name = L["HERB_MAGEROYAL"], skill = 0},
            {name = L["HERB_BRIARTHORN"], skill = 0},
            {name = L["HERB_BRUISEWEED"], skill = 0},
            {name = L["HERB_STRANGLEKELP"], skill = 85},
            {name = L["HERB_GRAVEMOSS"], skill = 105},
            {name = L["HERB_WILDSTEELBLOOM"], skill = 115},
            {name = L["HERB_KINGSBLOOD"], skill = 125},
            {name = L["HERB_FADELEAF"], skill = 150},
            {name = L["HERB_GOLDTHORN"], skill = 150},
            {name = L["HERB_LIFEROOT"], skill = 150},
            {name = L["HERB_KHADGARSWHISKER"], skill = 160},
            {name = L["HERB_DRAGONSTEETH"], skill = 195},
            {name = L["HERB_FIREBLOOM"], skill = 205},
            {name = L["HERB_PURPLELOTUS"], skill = 210},
            {name = L["HERB_ARTHASTEARS"], skill = 220},
            {name = L["HERB_SUNGRASS"], skill = 230},
            {name = L["HERB_BLINDWEED"], skill = 235},
            {name = L["HERB_GHOSTMUSHROOM"], skill = 245},
            {name = L["HERB_GROMSBLOOD"], skill = 250},
            {name = L["HERB_GOLDENSANSAM"], skill = 260},
            {name = L["HERB_DREAMFOIL"], skill = 270},
            {name = L["HERB_ICECAP"], skill = 270},
            {name = L["HERB_MOUNTAINSILVERSAGE"], skill = 280},
            {name = L["HERB_SORROWMOSS"], skill = 285},
            {name = L["HERB_BLACKLOTUS"], skill = 300},

            {name = "\n" .. L["HERB_FELWEED"], skill = 300},
            {name = L["HERB_DREAMINGGLORY"], skill = 315},
            {name = L["HERB_RAGVEIL"], skill = 325},
            {name = L["HERB_TEROCONE"], skill = 325},
            {name = L["HERB_FLAMECAP"], skill = 335},
            {name = L["HERB_ANCIENTLICHEN"], skill = 340},
            {name = L["HERB_NETHERBLOOM"], skill = 350},
            {name = L["HERB_NIGHTMAREVINE"], skill = 365},
            {name = L["HERB_MANATHISTLE"], skill = 375},

            {name = "\n" .. L["HERB_GOLDCLOVER"], skill = 350},
            {name = L["HERB_FIRETHORN"], skill = 360},
            {name = L["HERB_TIGERLILY"], skill = 375},
            {name = L["HERB_TALANDRASROSE"], skill = 385},
            {name = L["HERB_ADDERSTONGUE"], skill = 400},
            {name = L["HERB_LICHBLOOM"], skill = 425},
            {name = L["HERB_ICETHORN"], skill = 435},
            {name = L["HERB_FROSTLOTUS"], skill = 450},

            {name = "\n" .. L["HERB_AZSHARASVEIL"], skill = 425},
            {name = L["HERB_CINDERBLOOM"], skill = 425},
            {name = L["HERB_STORMVINE"], skill = 425},
            {name = L["HERB_HEARTBLOSSOM"], skill = 475},
            {name = L["HERB_WHIPTAIL"], skill = 500},
            {name = L["HERB_TWILIGHTJASMINE"], skill = 525},

            {name = "\n" .. L["HERB_GREENTEALEAF"], skill = 500},
            {name = L["HERB_RAINPOPPY"], skill = 525},
            {name = L["HERB_SILKWEED"], skill = 545},
            {name = L["HERB_GOLDENLOTUS"], skill = 550},
            {name = L["HERB_SHATOUCHED"], skill = 575},
            {name = L["HERB_SNOWLILY"], skill = 575},
            {name = L["HERB_FOOLSCAP"], skill = 600},
        },
    };

    local list = "";
    local cutoffLow = "";
    local cutoffHigh = "";

    if (nodes[profession]) then
        for index, entry in pairs(nodes[profession]) do
            local skip = false;
            local reqLevel = "";
            -- below the required skill to gather the node
            if ((skillLevel < entry.skill) and TRACKOMATIC_VARS['config']['gather_exclude_high']) then
                if ((GATHER_CUTOFFRED < 1) or ((entry.skill - skillLevel > GATHER_CUTOFFRED) and (GATHER_CUTOFFRED > 0))) then
                    cutoffHigh = "\n|cffff0000...|r";
                    skip = true;
                end
            elseif (((GATHER_CUTOFFGRAY > 0) and (skillLevel - (entry.skill + 100) >= GATHER_CUTOFFGRAY)) and TRACKOMATIC_VARS['config']['gather_exclude_low']) then
                cutoffLow = "|cffa0a0a0...|r\n";
                skip = true;
            end
            if (not skip) then
                local turnsYellow = entry.skill + 25;
                if (entry.turnsYellow) then
                    turnsYellow = entry.turnsYellow;
                end
                if (list ~= "") then list = list .. "\n"; end
                if (skillLevel < entry.skill) then
                    reqLevel = " " .. TrackOMatic_GetSkillColor(entry.skill, entry.skill, turnsYellow) .. "(" .. entry.skill .. ")|r";
                else
                    if (TRACKOMATIC_VARS['config']['advanced_gather_tooltips']) then
                        if (skillLevel < turnsYellow) then
                            reqLevel = " " .. TrackOMatic_GetSkillColor(turnsYellow, entry.skill, turnsYellow) .. "(" .. turnsYellow .. ")|r";
                        else
                            if (skillLevel < turnsYellow + 25) then
                                reqLevel = " " .. TrackOMatic_GetSkillColor(turnsYellow + 25, entry.skill, turnsYellow) .. "(" .. (turnsYellow + 25) .. ")|r";
                            else
                                if (skillLevel < turnsYellow + 75) then
                                    reqLevel = " " .. TrackOMatic_GetSkillColor(turnsYellow + 75, entry.skill, turnsYellow) .. "(" .. (turnsYellow + 75) .. ")|r";
                                end
                            end
                        end
                    end
                end
                list = list .. TrackOMatic_GetSkillColor(skillLevel, entry.skill, turnsYellow) .. entry.name .. "|r" .. reqLevel;
            end
        end
    end

    return cutoffLow .. list .. cutoffHigh;

end

--========================================
-- Get the color of a gatherable node
-- by skill level
--========================================
function TrackOMatic_GetSkillColor(level, required, turnsYellow)

    if (not turnsYellow) then
        turnsYellow = required + 25;
    end

    if (level >= (turnsYellow + 75)) then
        return "|cffa0a0a0";
    else
        if (level >= (turnsYellow + 25)) then
            return "|cff00ff00";
        else
            if (level >= turnsYellow) then
                return "|cffffff00";
            else
                if (level >= required) then
                    return "|cffffa000";
                else
                    return "|cffff0000";
                end
            end
        end
    end

end

--========================================
-- Add a profession to the tracker
--========================================
function TrackOMatic_AddProfession(name)

    local skillID, skillName = TrackOMatic_GetProfessionInfo(name);

    if (skillID == nil) then
        return;
    end

    if (TrackOMatic_AddToCategory("skill", {id = skillName})) then
        TrackOMatic_Message(string.format(L["TRACKER_ADD_PROFESSION"], skillName));
    end

end

--========================================
-- Remove a profession from the tracker
--========================================
function TrackOMatic_RemoveSkill(index, isSilent)
    local data = TrackOMatic_RemoveFromCategory("skill", index);
    if ((not isSilent) and data) then
        TrackOMatic_Message(string.format(L["TRACKER_REMOVE_PROFESSION"], data.id));
    end
end

--========================================
-- Update the professions tab of the
-- spellbook
--========================================
function TrackOMatic_UpdateProfessions()
    local prim1, prim2, arch, fishing, cooking, firstAid = GetProfessions();
    TrackOMatic_UpdateProfessionButton(PrimaryProfession1TrackButton, prim1);
    TrackOMatic_UpdateProfessionButton(PrimaryProfession2TrackButton, prim2);
    TrackOMatic_UpdateProfessionButton(SecondaryProfession1TrackButton, arch);
    TrackOMatic_UpdateProfessionButton(SecondaryProfession2TrackButton, fishing);
    TrackOMatic_UpdateProfessionButton(SecondaryProfession3TrackButton, cooking);
    TrackOMatic_UpdateProfessionButton(SecondaryProfession4TrackButton, firstAid);
    TrackOMatic_UpdateTracker();
end

--========================================
-- Update a button on the professions
-- tab of the spellbook
--========================================
function TrackOMatic_UpdateProfessionButton(button, index)
    if (index) then
        button:Show();
    else
        button:Hide();
    end
end

--========================================
-- Handler for hovering over a button on
-- the professions tab of the spellbook
--========================================
function TrackOMatic_AddProfessionButton_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    GameTooltip:SetText(L["TRACK_PROFESSION_TOOLTIP"], 1, 1, 1);
    GameTooltip:Show();
end

--========================================
-- Return whether a profession bar is
-- glowing based on current skill
--========================================
function TrackOMatic_NewProfessionRankAvailable(value, max)

    if ((value >= (max - 25)) and (max < TrackOMatic_GetProfessionCap()) and TRACKOMATIC_VARS['config']['profession_glow']) then
        return true;
    end
    return false;

end

--========================================
-- Return the current profession cap
-- based on the account's expansion level
--========================================
function TrackOMatic_GetProfessionCap()

    local exp = GetAccountExpansionLevel();
    local max = table.maxn(TRACKOMATIC_PROFESSION_CAPS);
    local idx = math.min(exp, max);

    return TRACKOMATIC_PROFESSION_CAPS[idx];

end
